@@ -22,23 +22,12 @@
 # deps: python-html python-image python-netclient python-misc python-pygtk python-mime python-json
 
 # todo:
-# add translation support?
-# download in seperate thread?
+# download logs
 # parse attributes
 # add "next waypoint" button
 # add description to displayed images
-# only redraw if position is in current screen
-# add favorite-places-feature: mark fav. places and return to them later
-# add favorite-caches-feature into new search tab
-# try to download caches in smaller portions
-
-# go to search results in case of search
-# don't display all caches on 'revert'
-# fix north indicator
-
-# to test:
-# new exporter
-# new search tab
+# add translation support?
+# download in seperate thread?
 
 
  
@@ -50,13 +39,11 @@ import geo
 import geocaching
 import gobject
 import gtk
-import gtk.gdk
 import gtk.glade
 import openstreetmap
 import os
 import pango
 import re
-#from advancedcaching import *
 
 
 class SimpleGui(object):
@@ -66,13 +53,19 @@ class SimpleGui(object):
     CACHE_SIZE = 20
     CLICK_RADIUS = 20
     TOO_MUCH_POINTS = 30
+    CACHES_ZOOM_LOWER_BOUND = 9
     CACHE_DRAW_SIZE = 10
     CACHE_DRAW_FONT = pango.FontDescription("Sans 4")
+    MESSAGE_DRAW_FONT = pango.FontDescription("Sans 5")
+    MESSAGE_DRAW_COLOR = gtk.gdk.color_parse('black')
+
+
     REDRAW_DISTANCE_TRACKING = 50 # distance from center of visible map in px
     REDRAW_DISTANCE_MINOR = 4 # distance from last displayed point in px
     DISTANCE_DISABLE_ARROW = 5 # meters
 
     MAX_NUM_RESULTS = 50
+    MAX_NUM_RESULTS_SHOW = 100
 
 
     ARROW_OFFSET = 1.0 / 3.0 # Offset to center of arrow, calculated as 2-x = sqrt(1^2+(x+1)^2)
@@ -130,7 +123,8 @@ class SimpleGui(object):
         self.userpointprovider = userpointprovider
                 
         self.format = geo.Coordinate.FORMAT_DM
-                
+
+        # @type self.current_cache geocaching.GeocacheCoordinate
         self.current_cache = None
                 
         self.current_target = None
@@ -279,10 +273,6 @@ class SimpleGui(object):
         listview.connect('extlistview-button-pressed', self.on_search_cache_clicked)
         xml.get_widget('scrolledwindow_search').add(listview)
                 
-        # Create the renderer used in the listview for coordinates
-                
-        #txtRdr        = gtk.CellRendererText()
-        #pixbufRdr = gtk.CellRendererPixbuf()
         (
          COL_COORD_NAME,
          COL_COORD_LATLON,
@@ -638,10 +628,103 @@ class SimpleGui(object):
                         d = openstreetmap.TileLoader(tile, self.ts.zoom, self, self.settings['download_map_path'], (i * dir_ew) * span_x + (j * dir_ns))
                         d.start()
         #print 'end draw map'
-                        
+
+    def __draw_marks_caches(self, coords):
+        xgc = self.xgc
+        draw_short = (len(coords) > self.TOO_MUCH_POINTS)
+
+        for c in coords: # for each geocache
+            radius = self.CACHE_DRAW_SIZE
+            if c.found:
+                color = self.COLOR_FOUND
+            elif c.type == geocaching.GeocacheCoordinate.TYPE_REGULAR:
+                color = self.COLOR_REGULAR
+            elif c.type == geocaching.GeocacheCoordinate.TYPE_MULTI:
+                color = self.COLOR_MULTI
+            else:
+                color = self.COLOR_DEFAULT
+
+            p = self.__coord2point(c)
+            xgc.set_rgb_fg_color(color)
+
+            if draw_short:
+                radius = radius / 2.0
+
+            if c.marked:
+                xgc.set_rgb_fg_color(self.COLOR_MARKED)
+                self.pixmap_marks.draw_rectangle(xgc, True, p[0] - radius, p[1] - radius, radius * 2, radius * 2)
+
+
+            xgc.set_rgb_fg_color(color)
+            xgc.line_width = 3
+            self.pixmap_marks.draw_rectangle(xgc, False, p[0] - radius, p[1] - radius, radius * 2, radius * 2)
+
+            if draw_short:
+                continue
+
+
+            # print the name?
+            if self.settings['options_show_name']:
+                layout = self.drawing_area.create_pango_layout(c.name)
+                layout.set_font_description(self.CACHE_DRAW_FONT)
+                self.pixmap_marks.draw_layout(xgc, p[0] + 3 + radius, p[1] - 3 - radius, layout)
+
+            # if we have a description for this cache...
+            if c.was_downloaded():
+                # draw something like:
+                # ----
+                # ----
+                # ----
+                # besides the icon
+                width = 6
+                dist = 2
+                pos_x = p[0] + radius + 3 + 1
+                pos_y = p[1] + 2
+                xgc.line_width = 1
+                for i in xrange(0, 3):
+                    self.pixmap_marks.draw_line(xgc, pos_x, pos_y + dist * i, pos_x + width, pos_y + dist * i)
+
+            # if this cache is the active cache
+            if self.current_cache != None and c.name == self.current_cache.name:
+                xgc.line_width = 3
+                xgc.set_rgb_fg_color(self.COLOR_CURRENT_CACHE)
+                radius = 7
+                self.pixmap_marks.draw_rectangle(xgc, False, p[0] - radius, p[1] - radius, radius * 2, radius * 2)
+
+            xgc.set_rgb_fg_color(self.COLOR_CACHE_CENTER)
+            xgc.line_width = 1
+            self.pixmap_marks.draw_line(xgc, p[0], p[1] - 2, p[0], p[1] + 3) #  |
+            self.pixmap_marks.draw_line(xgc, p[0] - 2, p[1], p[0] + 3, p[1]) # ---
+
+        # draw additional waypoints
+        # --> print description!
+        if self.current_cache != None and self.current_cache.get_waypoints() != None:
+            xgc.set_function(gtk.gdk.AND)
+            xgc.set_rgb_fg_color(self.COLOR_WAYPOINTS)
+            xgc.line_width = 1
+            radius = 4
+            num = 0
+            for w in self.current_cache.get_waypoints():
+                if w['lat'] != -1 and w['lon'] != -1:
+                    num = num + 1
+                    p = self.__coord2point(geo.Coordinate(w['lat'], w['lon']))
+                    self.pixmap_marks.draw_line(xgc, p[0], p[1] - 3, p[0], p[1] + 4) #  |
+                    self.pixmap_marks.draw_line(xgc, p[0] - 3, p[1], p[0] + 4, p[1]) # ---
+                    self.pixmap_marks.draw_arc(xgc, False, p[0] - radius, p[1] - radius, radius * 2, radius * 2, 0, 360 * 64)
+                    layout = self.drawing_area.create_pango_layout('')
+                    layout.set_markup('<i>%s</i>' % (w['id']))
+                    layout.set_font_description(self.CACHE_DRAW_FONT)
+                    self.pixmap_marks.draw_layout(xgc, p[0] + 3 + radius, p[1] - 3 - radius, layout)
+
+    def __draw_marks_message(self, message):
+        xgc = self.xgc
+        xgc.set_rgb_fg_color(self.MESSAGE_DRAW_COLOR)
+        layout = self.drawing_area.create_pango_layout(message)
+        layout.set_font_description(self.MESSAGE_DRAW_FONT)
+        self.pixmap_marks.draw_layout(xgc, 20, 20, layout)
+
     def __draw_marks(self):
             
-        #print "marking"
         xgc = self.xgc
         xgc.set_function(gtk.gdk.COPY)
         self.xgc.set_rgb_fg_color(gtk.gdk.color_parse('white'))
@@ -651,102 +734,20 @@ class SimpleGui(object):
         # draw geocaches
         #
 
-        if self.settings['options_hide_found']:
-            found = False
+        if self.ts.get_zoom() < self.CACHES_ZOOM_LOWER_BOUND:
+            self.__draw_marks_message('Zoom in to see geocaches.')
         else:
-            found = None
-        coords = self.pointprovider.get_points_filter(self.get_visible_area(), found)
-        if len(coords) < self.pointprovider.MAX_RESULTS:
-
-            draw_short = (len(coords) > self.TOO_MUCH_POINTS)
-
-            xgc.set_function(gtk.gdk.COPY)
-            #xgc.set_dashes(1, (2, 2))
-            #xgc.line_style = gtk.gdk.LINE_ON_OFF_DASH
-
-            for c in coords: # for each geocache
-                radius = self.CACHE_DRAW_SIZE
-                if c.found:
-                    color = self.COLOR_FOUND
-                elif c.type == geocaching.GeocacheCoordinate.TYPE_REGULAR:
-                    color = self.COLOR_REGULAR
-                elif c.type == geocaching.GeocacheCoordinate.TYPE_MULTI:
-                    color = self.COLOR_MULTI
-                else:
-                    color = self.COLOR_DEFAULT
-
-                p = self.__coord2point(c)
-                xgc.set_rgb_fg_color(color)
-
-                if draw_short:
-                    radius = radius / 2.0
-
-                if c.marked:
-                    xgc.set_rgb_fg_color(self.COLOR_MARKED)
-                    self.pixmap_marks.draw_rectangle(xgc, True, p[0] - radius, p[1] - radius, radius * 2, radius * 2)
-
-
-                xgc.set_rgb_fg_color(color)
-                xgc.line_width = 3
-                self.pixmap_marks.draw_rectangle(xgc, False, p[0] - radius, p[1] - radius, radius * 2, radius * 2)
-
-                if draw_short:
-                    continue
-
-
-                # print the name?
-                if self.settings['options_show_name']:
-                    layout = self.drawing_area.create_pango_layout(c.name)
-                    layout.set_font_description(self.CACHE_DRAW_FONT)
-                    self.pixmap_marks.draw_layout(xgc, p[0] + 3 + radius, p[1] - 3 - radius, layout)
-
-                # if we have a description for this cache...
-                if c.was_downloaded():
-                    # draw something like:
-                    # ----
-                    # ----
-                    # ----
-                    # besides the icon
-                    width = 6
-                    dist = 2
-                    pos_x = p[0] + radius + 3 + 1
-                    pos_y = p[1] + 2
-                    xgc.line_width = 1
-                    for i in xrange(0, 3):
-                        self.pixmap_marks.draw_line(xgc, pos_x, pos_y + dist * i, pos_x + width, pos_y + dist * i)
-
-                # if this cache is the active cache
-                if self.current_cache != None and c.name == self.current_cache.name:
-                    xgc.line_width = 3
-                    xgc.set_rgb_fg_color(self.COLOR_CURRENT_CACHE)
-                    radius = 7
-                    self.pixmap_marks.draw_rectangle(xgc, False, p[0] - radius, p[1] - radius, radius * 2, radius * 2)
-
-                xgc.set_rgb_fg_color(self.COLOR_CACHE_CENTER)
-                xgc.line_width = 1
-                self.pixmap_marks.draw_line(xgc, p[0], p[1] - 2, p[0], p[1] + 3) #  |
-                self.pixmap_marks.draw_line(xgc, p[0] - 2, p[1], p[0] + 3, p[1]) # ---
-
-            # draw additional waypoints
-            # --> print description!
-            if self.current_cache != None and self.current_cache.get_waypoints() != None:
-                xgc.set_function(gtk.gdk.AND)
-                xgc.set_rgb_fg_color(self.COLOR_WAYPOINTS)
-                xgc.line_width = 1
-                radius = 4
-                num = 0
-                for w in self.current_cache.get_waypoints():
-                    if w['lat'] != -1 and w['lon'] != -1:
-                        num = num + 1
-                        p = self.__coord2point(geo.Coordinate(w['lat'], w['lon']))
-                        self.pixmap_marks.draw_line(xgc, p[0], p[1] - 3, p[0], p[1] + 4) #  |
-                        self.pixmap_marks.draw_line(xgc, p[0] - 3, p[1], p[0] + 4, p[1]) # ---
-                        self.pixmap_marks.draw_arc(xgc, False, p[0] - radius, p[1] - radius, radius * 2, radius * 2, 0, 360 * 64)
-                        layout = self.drawing_area.create_pango_layout('')
-                        layout.set_markup('<i>%s</i>' % (w['id']))
-                        layout.set_font_description(self.CACHE_DRAW_FONT)
-                        self.pixmap_marks.draw_layout(xgc, p[0] + 3 + radius, p[1] - 3 - radius, layout)
 
+            if self.settings['options_hide_found']:
+                found = False
+            else:
+                found = None
+            coords = self.pointprovider.get_points_filter(self.get_visible_area(), found, self.MAX_NUM_RESULTS_SHOW)
+            if len(coords) >= self.MAX_NUM_RESULTS_SHOW:
+                self.__draw_marks_message('Too much geocaches to display.')
+            else:
+                self.__draw_marks_caches(coords)
+            
                         
         #
         # next, draw the user defined points
@@ -896,21 +897,12 @@ class SimpleGui(object):
                 
                         
                 
-    def load_images(self):
+    def __load_images(self):
         if self.current_cache == None:
-            self.update_cache_image(reset = True)
+            self.__update_cache_image(reset = True)
             return
-        try:
-            files = os.listdir(self.settings['download_output_dir'])
-            images = []
-            for f in files:
-                if f.startswith('%s-image' % self.current_cache.name):
-                    images.append(os.path.join(self.settings['download_output_dir'], f))
-                                
-            self.images = images
-        except Exception as e:
-            print "Could not prepare images: %s" % e
-        self.update_cache_image(reset = True)
+        self.images = self.current_cache.get_images().items()
+        self.__update_cache_image(reset = True)
 
     def on_download_clicked(self, widget):
         self.do_events()
@@ -1005,9 +997,14 @@ class SimpleGui(object):
 
             # if we are tracking and we have not moved out of the center
             # or if we are not tracking the user
-            # in each case, if we have moved far enough since last draw, redraw just the marks
+            # in either case, if we have moved far enough since last draw, redraw just the marks
             if dist_from_last > self.REDRAW_DISTANCE_MINOR ** 2:
-                self.redraw_marks()
+                a, b = self.get_visible_area()
+                if self.gps_data.position.lat > min(a.lat, b.lat) \
+                    and self.gps_data.position.lat < max(a.lat, b.lat) \
+                    and self.gps_data.position.lon > min(a.lon, b.lon) \
+                    and self.gps_data.position.lon < max(a.lon, b.lon):
+                        self.redraw_marks()
                 # update last position, as it is now drawed
                 self.gps_last_position = (x, y)
             return
@@ -1024,16 +1021,16 @@ class SimpleGui(object):
         
     def on_image_next_clicked(self, something):
         if len(self.images) == 0:
-            self.update_cache_image(reset = True)
+            self.__update_cache_image(reset = True)
             return
         self.image_no += 1
         self.image_no %= len(self.images)
-        self.update_cache_image()
+        self.__update_cache_image()
                 
         
     def on_image_zoom_clicked(self, something):
         self.image_zoomed = not self.image_zoomed
-        self.update_cache_image()
+        self.__update_cache_image()
 
     def on_label_fieldnotes_mapped(self, widget):
         self.__check_notes_save()
@@ -1243,45 +1240,32 @@ class SimpleGui(object):
     def on_zoomout_clicked(self, widget):
         self.zoom(-1)
                 
-    def update_cache_image(self, reset = False):
+    def __update_cache_image(self, reset = False):
         if reset:
-            self.image_cache.set_from_stock(gtk.STOCK_GO_FORWARD, -1)
-            self.image_cache_caption.set_text("click 'next' to see images")
-            return
-        try:
-            mw, mh = self.scrolledwindow_image.get_allocation().width - 10, self.scrolledwindow_image.get_allocation().height - 10
-            if self.current_cache == None or len(self.images) <= self.image_no:
-                self.update_cache_image(True)
+            self.image_zoomed = False
+            self.image_no = 0
+            if len(self.images) == 0:
+                self.image_cache.set_from_stock(gtk.STOCK_CANCEL   , -1)
+                self.image_cache_caption.set_text("There's nothing to see here.")
                 return
-            filename = os.path.join(self.settings['download_output_dir'], self.images[self.image_no])
+        try:
+            if self.current_cache == None or len(self.images) <= self.image_no:
+                self.__update_cache_image(True)
+                return
+            filename = os.path.join(self.settings['download_output_dir'], self.images[self.image_no][0])
             if not os.path.exists(filename):
                 self.image_cache_caption.set_text("not found: %s" % filename)
                 self.image_cache.set_from_stock(gtk.STOCK_GO_FORWARD, -1)
                 return
-                        
-            pb = gtk.gdk.pixbuf_new_from_file(filename)
-                
+            
             if not self.image_zoomed:
-                w = float(pb.get_width())
-                h = float(pb.get_height())
-                if w > mw:
-                    factor = mw / w
-                    w = w * factor
-                    h = h * factor
-                if h > mh:
-                    factor = mh / h
-                    w = w * factor
-                    h = h * factor
-                pb = pb.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
-                
+                mw, mh = self.scrolledwindow_image.get_allocation().width - 10, self.scrolledwindow_image.get_allocation().height - 10
+                pb = gtk.gdk.pixbuf_new_from_file_at_size(filename, mw, mh)
+            else:
+                pb = gtk.gdk.pixbuf_new_from_file(filename)
                         
             self.image_cache.set_from_pixbuf(pb)
-            imgs = self.current_cache.get_images()
-            print imgs
-            if self.images[self.image_no] in imgs.keys():
-                caption = imgs[self.images[self.image_no]]
-            else:
-                caption = "(no caption)"
+            caption = self.images[self.image_no][1]
 
             self.image_cache_caption.set_text("<b>%d</b> %s" % (self.image_no, caption))
             self.image_cache_caption.set_use_markup(True)
@@ -1471,7 +1455,7 @@ class SimpleGui(object):
             self.cache_elements['log_date'].set_text('fieldnote date: not set')
 
         # Load images
-        self.load_images()
+        self.__load_images()
         self.image_no = 0
         if len(self.images) > 0:
             showdesc += "\n[%d image(s) available]" % len(self.images)
